//---------------------------------------------------------------------------

#pragma hdrstop

#include "ArmScope.h"
#include "armscope_cmd.h"
#include "..\miniscope\device\MiniscopeDevice.h"
#include "ThreadComm.h"
#include "usb.h"
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "libusb.lib"

ArmScope armScope;
struct usb_device *my_dev = NULL;
struct usb_dev_handle *usb_handle = NULL;

#define VENDOR_ID   0x03EB      //0x03EB
#define PRODUCT_ID  0x6130    //0x2129

static struct usb_device *device_init(void)
{
    struct usb_bus *usb_bus;
    struct usb_device *dev;
    usb_init();
//    armScope.log("USB: Enumerating devices...");
    usb_find_busses();
    usb_find_devices();
    for (usb_bus = usb_get_busses(); usb_bus;
            usb_bus = usb_bus->next)
    {
        for (dev = usb_bus->devices; dev;
                dev = dev->next)
        {
            if ((dev->descriptor.idVendor == VENDOR_ID) && (dev->descriptor.idProduct == PRODUCT_ID))
            {
//                    Log("Device found\r\n");
                return dev;
            }
        }
    }
    return NULL;
}


ArmScope::ArmScope()
{
    bDeviceOpened = false;
    bStopRequest = false;
    callbackData = NULL;
    callbackLog = NULL;
    callbackConnect = NULL;
	threadComm = new ThreadComm(false);
	threadComm->SetContinuousTrigger(false);
}

ArmScope::~ArmScope()
{
    if (threadComm)
    {
        threadComm->Terminate();
    }
}

void ArmScope::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
    va_list ap;
    va_start(ap, lpData);
    vsnprintf(buf, sizeof(buf), lpData, ap);
    va_end(ap);
    buf[sizeof(buf)-1] = 0;

    if (callbackLog)
        callbackLog(buf);
}

int ArmScope::Connect(void)
{
    char txtbuf[200];
    my_dev = device_init();
    if (my_dev == NULL)
    {
        ConnectCb(DEVICE_DISCONNECTED, "Device not found. Connect device and restart program.");
        log( "Device not found. Connect miniscope and restart application." );
        return 1;
    }
    else
    {
        //open device
        usb_handle = usb_open( my_dev );
        if (usb_handle == NULL)
        {
            ConnectCb(DEVICE_DISCONNECTED, "Error opening device handle.");
            log("device opening failed");
            return 2;
        }
        else
        {
            //set configuration
            int ret;
            if ( ( ret = usb_set_configuration( usb_handle, 1 ) ) < 0 )
            {
                ConnectCb(DEVICE_DISCONNECTED, "Failed to set device interface configuration.");
                log("usb config set failed with status %d", ret);
                return 3;
            }
            if ( ( ret = usb_claim_interface( usb_handle, 1 ) ) < 0 )   //chyba interfejs 1 - ten od BULK
            {
                ConnectCb(DEVICE_DISCONNECTED, "Failed to claim USB interface.");
                Disconnect();
                log("FAIL: %s", usb_strerror());
                return 4;
            }
            else
            {
                txtbuf[0] = NULL;
                char dummybuf[5000];
                int cnt = 0;
                while (!strstr(txtbuf,"VDSO") && (cnt++ < 5))
                {
                    log("Flushing...");
                    ret = usb_bulk_read( usb_handle, 0x82, dummybuf, sizeof(dummybuf), 100 );
                    ret = usb_bulk_read( usb_handle, 0x82, dummybuf, sizeof(dummybuf), 100 );
                    ret = usb_bulk_read( usb_handle, 0x82, dummybuf, sizeof(dummybuf), 100 );
                    ret = usb_bulk_read( usb_handle, 0x82, dummybuf, sizeof(dummybuf), 100 );
                    ret = usb_bulk_read( usb_handle, 0x82, dummybuf, sizeof(dummybuf), 100 );

                    log("Sending ID request");
                    if (threadComm->SendCmd(CMD_FUNCTION, CMD_FUNCTION_IDENTIFY) == 0)
                        usb_bulk_read( usb_handle, 0x82, txtbuf, 16, 2000 );
                    else
                        return 5;
                }
                if (cnt == 6)
                {
                    ConnectCb(DEVICE_DISCONNECTED, "Invalid device ID received");
                    log("Invalid device ID");
                    Disconnect();
                    return 6;
                }

                log ("Device connected, ID: %s", txtbuf);
                ConnectCb(DEVICE_CONNECTED, txtbuf);
                /*
                				Form1->StatusBar1->Panels->Items[3]->Text=txtbuf;
                				ReadThread = new TRead(false);
                				//przeslanie konfiguracji do urzadzenia
                				USBsend(0x01, 20-TimeBaseTrackBar->Position);   //podstawa czasu
                				//czulosc
                				USBsend(0x04, SensitivityTrackBar->Position);
                				//rodzaj sprzezenia
                				USBsend(0x05, SpeedButton6->Down?0x02:0x01);
                				//rodzaj wyzwalania
                				USBsend(0x05, SpeedButton9->Down?0x04:0x06);
                				//poziom wyzwalania
                				USBsend(0x03, TriggerTrackBar->Position);
                				//aktualizacja wielkosci bufora - symulacja wywolania z kontrolki
                				OnBufferSizeChange(NULL);
                				last_par.NSamples = NSamples;
                				ActivateControls(1);
                */
                threadComm->callbackLog = callbackLog;
				threadComm->callbackData = callbackData;
				threadComm->callbackTrigger = callbackTrigger;
                threadComm->Execute();
            }
        }
    }
    return 0;
}

int ArmScope::Disconnect(void)
{
	if (threadComm)
	{
		threadComm->SetContinuousTrigger(false);
		Sleep(100);
	}
	if (usb_handle == NULL)
        return 0;
    usb_close(usb_handle);
    usb_handle = NULL;
    return 0;
}

int ArmScope::ManualTrigger(void)
{
	if (!threadComm)
		return -1;
	return threadComm->SendCmd(CMD_FUNCTION, CMD_FUNCTION_TRIGGER_EXECUTE);
}

int ArmScope::SetTriggerType(enum E_TRIGGER_TYPE type)
{
	if (!threadComm)
		return -1;
	threadComm->SetContinuousTrigger(false);		
	enum E_SUBCOMMAND_FUNCTION_TYPE cmd;
	if (type == TRIGGER_MANUAL)
	{
		cmd = CMD_FUNCTION_TRIGGER_MANUAL;
	}
	else if (type == TRIGGER_SLOPE_UP)
		cmd = CMD_FUNCTION_TRIGGER_SLOPE_RAISING;
	else if (type == TRIGGER_SLOPE_DOWN)
		cmd = CMD_FUNCTION_TRIGGER_SLOPE_FALLING;
	else if (type == TRIGGER_CONTINUOUS)
	{
		cmd = CMD_FUNCTION_TRIGGER_MANUAL;
		threadComm->SetContinuousTrigger(true);
	}
	else
		return -2;
	return threadComm->SendCmd(CMD_FUNCTION, cmd);
}

int ArmScope::SetCouplingType(enum E_COUPLING_TYPE type)
{
	if (!threadComm)
		return -1;
	enum E_SUBCOMMAND_FUNCTION_TYPE cmd;
	if (type == COUPLING_DC)
		cmd = CMD_FUNCTION_COUPLING_DC;
	else if (type == COUPLING_AC)
		cmd = CMD_FUNCTION_COUPLING_AC;
	else
		return -2;
	return threadComm->SendCmd(CMD_FUNCTION, cmd);
}

int ArmScope::SetBufferSize(int size)
{
	if (!threadComm)
		return -1;
	unsigned char val;
	//512 samples for 0x00, 1024 samples, 2048 samples,
	// 4096 samples, 8192 samples and 14336 samples for 0x05
	int samples;
	if (size >= 14336)
	{
		val = 0x05;
		samples = 14336;
	}
	else if (size >= 8192)
	{
		val = 0x04;
		samples = 8192;
	}
	else if (size >= 4096)
	{
		val = 0x03;
		samples = 4096;
	}
	else if (size >= 2048)
	{
		val = 0x02;
		samples = 2048;
	}
	else if (size >= 1024)
	{
		val = 0x01;
		samples = 1024;
	}
	else
	{
		val = 0x00;
		samples = 512;
	}
	threadComm->SetBufferSize(samples);
	return threadComm->SendCmd(CMD_SET_CAPTURE_BUFFER_SIZE, val);
}

int ArmScope::SetSensitivity(int id)
{
	if (id < 0 || id > 5)
		return -1;
	return threadComm->SendCmd(CMD_SET_SENSITIVITY, id + 1);
}

int ArmScope::SetSampling(int id)
{
	if (id < 0 || id > 9)
		return -1;
	return threadComm->SendCmd(CMD_SET_SAMPLING_FREQ, id);
}

int ArmScope::SetTriggerLevel(int level)
{
	if (level < -100 || level > 100)
		return -1;
	int val = 255 - ((255 * (level+100))/200);
	return threadComm->SendCmd(CMD_SET_TRIGGER_LEVEL, val);
}

